/*********************************************************************
Features: RFID TARGET launch program
Description: This program uses a timed two seconds transmitter ID
once finished launch into low power mode, low power current 3uA
ID 6 byte length
**********************************************************************/

#include <reg24lu1.h>
#include <stdint.h>
#include "API.h"
#define p0dir 0xf0
#define INTERRUPT_RFIRQ 9
#define INTERRUPT_TICK 13

#define TX_ADR_WIDTH 5		// RF transceiver 5 bytes of address
#define TX_PLOAD_WIDTH 20	// packet length is 20 bytes

uint8_t const TX_ADDRESS[TX_ADR_WIDTH] = { 0x34, 0x56, 0x78, 0x90, 0x12 };	// define RF transceiver address

uint8_t data  id_buf[TX_PLOAD_WIDTH] = { 0xff, 0x01, 0x02, 0x03, 0x04, 0x05 };

uint8_t bdata sta;
sbit          RX_DR = sta ^ 6;
sbit          TX_DS = sta ^ 5;
sbit          MAX_RT = sta ^ 4;

/**************************************************
Function: Delay
**************************************************/
void
delay( uint16_t x )
{
  uint16_t      i, j;

  for ( i = 0; i < x; i++ )
  {
    j = 108;
    while ( j-- );
  }
}

/**************************************************
Function: Hardware SPI read and write
**************************************************/
uint8_t
SPI_RW( uint8_t value )
{
  RFDAT = value;
  RFSPIF = 0;			// IMPORTANT Clear RF SPI ready flag!!
  // After data written to RFDAT ..
  while ( !RFSPIF ) ;		// wait for byte transfer finished
  ;
  return RFDAT;			// return SPI read value
}

/**************************************************
Function: write RF register, read RF state value
**************************************************/
uint8_t
SPI_RW_Reg( uint8_t reg, uint8_t value )
{
  uint8_t       status;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  SPI_RW( value );		// write data
  RFCSN = 1;

  return ( status );		// returns the RF state value
}

/**************************************************
Function: Reading RF register
**************************************************/
uint8_t
SPI_Read( uint8_t reg )
{
  uint8_t       reg_val;

  RFCSN = 0;
  SPI_RW( reg );		// Select the RF register
  reg_val = SPI_RW( 0 );	// read data
  RFCSN = 1;

  return ( reg_val );		// returns the RF state value
}

/**************************************************
Function: Multi-byte data buffer is written to RF register
**************************************************/
uint8_t
SPI_Write_Buf( uint8_t reg, uint8_t * pBuf, uint8_t bytes )
{
  uint8_t       status, byte_ctr;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  for ( byte_ctr = 0; byte_ctr < bytes; byte_ctr++ )	// write data connection
    SPI_RW( *pBuf++ );
  RFCSN = 1;
  return ( status );		// returns the RF state value
}

/**************************************************
Function: Set to power-down mode
**************************************************/
/**************************************************
Function: Set to transmit mode
**************************************************/
void
TX_Mode( void )
{
  SPI_RW_Reg( WRITE_REG + CONFIG, 0x0e );	// on power, CRC is 2 bytes, the receive mode, an interrupt is generated to allow RX_DR
  SPI_Write_Buf( WR_TX_PLOAD, id_buf, TX_PLOAD_WIDTH );	// write data to the FIFO
  RFCE = 1;			// start the launch
  delay( 100 );			// transmit pulse
  RFCE = 0;
}

/**************************************************
Function: RF initialization
**************************************************/
void
rf_init( void )
{
  RFCE = 0;			// RF Close
  RFCKEN = 1;			// start the RF clock
  RF = 1;			// allows the RF interrupt

  delay( 1000 );

  SPI_Write_Buf( WRITE_REG + TX_ADDR, TX_ADDRESS, TX_ADR_WIDTH );	// Set transmitter address length
  SPI_Write_Buf( WRITE_REG + RX_ADDR_P0, TX_ADDRESS, TX_ADR_WIDTH );	// Set the receiver address length

  SPI_RW_Reg( WRITE_REG + EN_AA, 0x01 );	// start automatic answer function
  SPI_RW_Reg( WRITE_REG + EN_RXADDR, 0x01 );	// PIPE0 receive data
  SPI_RW_Reg( WRITE_REG + SETUP_RETR, 0x1a );	// automatic repeat 10 times
  SPI_RW_Reg( WRITE_REG + RF_CH, 40 );	// RF frequency 2440MHz
  SPI_RW_Reg( WRITE_REG + RF_SETUP, 0x0f );	// transmit power 0dBm, the transmission rate 2Mbps,
  SPI_RW_Reg( WRITE_REG + RX_PW_P0, TX_PLOAD_WIDTH );	// PIPE0 receive packet length
}

/**************************************************
Function: RF interrupt service routine
**************************************************/
void
RF_IRQ( void )
interrupt     INTERRUPT_RFIRQ
{
  sta = SPI_Read( STATUS );	// read status values
  SPI_RW_Reg( WRITE_REG + STATUS, 0x70 );	// clear all interrupt flags
}
/**************************************************
Function: main
**************************************************/
void
main( void )
{
  CLKCTL = 0;
  RFCE = 0;
  RFCTL = 0X10;
  rf_init(  );			// RF initialization
  EA = 1;			// enable interrupt
  while ( 1 )
  {
    TX_Mode(  );		// transmit data
    while ( !( TX_DS | MAX_RT ) ) ;	// wait for transmit end
    sta = 0;
    delay( 100 );
  }
}
