/*********************************************************************
Features: RFID READER receiving program
Description: Receive data display lights flash once,
and displays the ID value through the serial port on
**********************************************************************/

#include <reg24lu1.h>
#include <stdint.h>
#include "API.h"
#define p0dir 0xd8
#define p1dir 0xff
#define INTERRUPT_RFIRQ 9
#define TX_ADR_WIDTH 5		// RF transceiver 5 bytes of address
#define TX_PLOAD_WIDTH 20	// packet length is 20 bytes
#define CE_LOW () do {RFCE = 0;} while (0)
uint8_t const TX_ADDRESS[TX_ADR_WIDTH] = { 0x34, 0x56, 0x78, 0x90, 0x12 };	// define RF transceiver address

uint8_t data  rx_buf[TX_PLOAD_WIDTH];
uint8_t bdata sta;
sbit          RX_DR = sta ^ 6;
sbit          TX_DS = sta ^ 5;
sbit          MAX_RT = sta ^ 4;
/**************************************************
Function: Delay
**************************************************/
void
delay( uint16_t x )
{
  uint16_t      i, j;
  i = 0;
  for ( i = 0; i < x; i++ )
  {
    j = 108;
    while ( j-- ) ;
  }
}

/**************************************************
Function: Hardware SPI read and write
**************************************************/
uint8_t
SPI_RW( uint8_t value )
{
  RFDAT = value;
  RFSPIF = 0;			// IMPORTANT Clear RF SPI ready flag!!
  // After data written to RFDAT ..
  while ( !RFSPIF ) ;		// wait for byte transfer finished
  ;
  return RFDAT;			// return SPI read value
}

/**************************************************
Function: write RF register, read RF state value
**************************************************/
uint8_t
SPI_RW_Reg( uint8_t reg, uint8_t value )
{
  uint8_t       status;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  SPI_RW( value );		// write data
  RFCSN = 1;
  return ( status );		// returns the RF state value
}

/**************************************************
Function: Reading RF register
**************************************************/
uint8_t
SPI_Read( uint8_t reg )
{
  uint8_t       reg_val;

  RFCSN = 0;
  SPI_RW( reg );		// Select the RF register
  reg_val = SPI_RW( 0 );	// read data
  RFCSN = 1;
  return ( reg_val );		// returns the RF state value
}

/**************************************************
Features: Multi-byte read RF data to the buffer register
**************************************************/
uint8_t
SPI_Read_Buf( uint8_t reg, uint8_t * pBuf, uint8_t bytes )
{
  uint8_t       status, byte_ctr;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register

  for ( byte_ctr = 0; byte_ctr < bytes; byte_ctr++ )
    pBuf[byte_ctr] = SPI_RW( 0 );	// read data connection
  RFCSN = 1;
  return ( status );		// returns the RF state value
}

/**************************************************
Function: Multi-byte data buffer is written to RF register
**************************************************/
uint8_t
SPI_Write_Buf( uint8_t reg, uint8_t * pBuf, uint8_t bytes )
{
  uint8_t       status, byte_ctr;

  RFCSN = 0;
  status = SPI_RW( reg );	// Select the RF register
  for ( byte_ctr = 0; byte_ctr < bytes; byte_ctr++ )	// write data connection
    SPI_RW( *pBuf++ );
  RFCSN = 1;
  return ( status );		// returns the RF state value
}

/**************************************************
Function: set to receive mode
**************************************************/
void
RX_Mode( void )
{
  RFCE = 0;
  SPI_RW_Reg( WRITE_REG + CONFIG, 0x0f );	// on power, CRC is 2 bytes, the receive mode, an interrupt is generated to allow RX_DR
  RFCE = 1;			// start receiving mode
}

/**************************************************
Function: RF initialization
**************************************************/
void
rf_init( void )
{
  RFCE = 0;			// RF Close
  RFCKEN = 1;			// start the RF clock
  RF = 1;			// allows the RF interrupt
  delay( 10 );
  SPI_Write_Buf( WRITE_REG + TX_ADDR, TX_ADDRESS, TX_ADR_WIDTH );	// Set transmitter address length
  SPI_Write_Buf( WRITE_REG + RX_ADDR_P0, TX_ADDRESS, TX_ADR_WIDTH );	// Set the receiver address length
  SPI_RW_Reg( WRITE_REG + EN_AA, 0x01 );	// start automatic answer function
  SPI_RW_Reg( WRITE_REG + EN_RXADDR, 0x01 );	// PIPE0 receive data
  SPI_RW_Reg( WRITE_REG + SETUP_RETR, 0x1a );	// automatic repeat 10 times
  SPI_RW_Reg( WRITE_REG + RF_CH, 40 );	// RF frequency 2440MHz
  SPI_RW_Reg( WRITE_REG + RF_SETUP, 0x0f );	// transmit power 0dBm, the transmission rate 2Mbps,
  SPI_RW_Reg( WRITE_REG + RX_PW_P0, TX_PLOAD_WIDTH );	// PIPE0 receive packet length
}

/**************************************************
Function: RF interrupt service routine
**************************************************/
void
RF_IRQ( void )
interrupt     INTERRUPT_RFIRQ
{
  sta = SPI_Read( STATUS );	// read status values

  if ( RX_DR )
  {
    SPI_Read_Buf( RD_RX_PLOAD, rx_buf, TX_PLOAD_WIDTH );	// read data FIFO
    SPI_RW_Reg( FLUSH_RX, 0 );	// Clear the RX FIFO
  }

  SPI_RW_Reg    ( WRITE_REG + STATUS, 0x70 );	// clear all interrupt flags
}

/**************************************************
Function: serial port initialization subroutine
Description: baud 19.2K, using the internal baud rate generator
**************************************************/
/**************************************************
Function: send 1 byte of data to the serial port
**************************************************/
/**************************************************
Function: I / O port initialization
**************************************************/
/**************************************************
Function: main
**************************************************/
void
main( void )
{
  CLKCTL = 0;
  RFCE = 0;
  RFCTL = 0x10;
  rf_init(  );			// RF initialization
  EA = 1;			// enable interrupt
  RX_Mode(  );			// receive mode
  while ( 1 )
  {
    if ( RX_DR )		// data has been received
    {
      sta = 0;
      delay( 1000 );
    }
  }
}
